@@ -15,6 +15,8 @@ const POPUP_APPICON_SIZE = 96;
 const POPUP_SCROLL_TIME = 0.10; // seconds
 const POPUP_FADE_TIME = 0.1; // seconds
 
+const APP_ICON_HOVER_TIMEOUT = 200; // milliseconds
+
 const DISABLE_HOVER_TIMEOUT = 500; // milliseconds
 
 const THUMBNAIL_DEFAULT_SIZE = 256;
@@ -34,7 +36,8 @@ function AltTabPopup() {
 AltTabPopup.prototype = {
     _init : function() {
         this.actor = new Shell.GenericContainer({ name: 'altTabPopup',
-                                                    reactive: true });
+                                                  reactive: true,
+                                                  visible: false });
 
         this.actor.connect('get-preferred-width', Lang.bind(this, this._getPreferredWidth));
         this.actor.connect('get-preferred-height', Lang.bind(this, this._getPreferredHeight));
@@ -49,6 +52,8 @@ AltTabPopup.prototype = {
         this._thumbnailTimeoutId = 0;
         this._motionTimeoutId = 0;
 
+        this.thumbnailsVisible = false;
+
         // Initially disable hover so we ignore the enter-event if
         // the switcher appears underneath the current pointer location
         this._disableHover();
@@ -126,13 +131,13 @@ AltTabPopup.prototype = {
             return false;
         this._haveModal = true;
 
-        this._keyPressEventId = global.stage.connect('key-press-event', Lang.bind(this, this._keyPressEvent));
-        this._keyReleaseEventId = global.stage.connect('key-release-event', Lang.bind(this, this._keyReleaseEvent));
+        this.actor.connect('key-press-event', Lang.bind(this, this._keyPressEvent));
+        this.actor.connect('key-release-event', Lang.bind(this, this._keyReleaseEvent));
 
         this.actor.connect('button-press-event', Lang.bind(this, this._clickedOutside));
         this.actor.connect('scroll-event', Lang.bind(this, this._onScroll));
 
-        this._appSwitcher = new AppSwitcher(apps);
+        this._appSwitcher = new AppSwitcher(apps, this);
         this.actor.add_actor(this._appSwitcher.actor);
         this._appSwitcher.connect('item-activated', Lang.bind(this, this._appActivated));
         this._appSwitcher.connect('item-entered', Lang.bind(this, this._appEntered));
@@ -365,15 +370,18 @@ AltTabPopup.prototype = {
     },
 
     destroy : function() {
-        Tweener.addTween(this.actor,
-                         { opacity: 0,
-                           time: POPUP_FADE_TIME,
-                           transition: 'easeOutQuad',
-                           onComplete: Lang.bind(this,
-                               function() {
-                                   this.actor.destroy();
-                               })
-                         });
+        if (this.actor.visible) {
+            Tweener.addTween(this.actor,
+                             { opacity: 0,
+                               time: POPUP_FADE_TIME,
+                               transition: 'easeOutQuad',
+                               onComplete: Lang.bind(this,
+                                   function() {
+                                       this.actor.destroy();
+                                   })
+                             });
+        } else
+            this.actor.destroy();
     },
 
     _onDestroy : function() {
@@ -383,11 +391,6 @@ AltTabPopup.prototype = {
         if (this._thumbnails)
             this._destroyThumbnails();
 
-        if (this._keyPressEventId)
-            global.stage.disconnect(this._keyPressEventId);
-        if (this._keyReleaseEventId)
-            global.stage.disconnect(this._keyReleaseEventId);
-
         if (this._motionTimeoutId != 0)
             Mainloop.source_remove(this._motionTimeoutId);
         if (this._thumbnailTimeoutId != 0)
@@ -458,11 +461,15 @@ AltTabPopup.prototype = {
     },
 
     _destroyThumbnails : function() {
-        Tweener.addTween(this._thumbnails.actor,
+        let thumbnailsActor = this._thumbnails.actor;
+        Tweener.addTween(thumbnailsActor,
                          { opacity: 0,
                            time: THUMBNAIL_FADE_TIME,
                            transition: 'easeOutQuad',
-                           onComplete: function() { this.destroy(); }
+                           onComplete: Lang.bind(this, function() {
+                                                            thumbnailsActor.destroy();
+                                                            this.thumbnailsVisible = false;
+                                                        })
                          });
         this._thumbnails = null;
     },
@@ -478,7 +485,8 @@ AltTabPopup.prototype = {
         Tweener.addTween(this._thumbnails.actor,
                          { opacity: 255,
                            time: THUMBNAIL_FADE_TIME,
-                           transition: 'easeOutQuad'
+                           transition: 'easeOutQuad',
+                           onComplete: Lang.bind(this, function () { this.thumbnailsVisible = true; })
                          });
     }
 };
@@ -592,16 +600,20 @@ SwitcherList.prototype = {
         this._list.add_actor(bbox);
 
         let n = this._items.length;
-        bbox.connect('clicked', Lang.bind(this, function () {
-                                               this._itemActivated(n);
-                                          }));
-        bbox.connect('enter-event', Lang.bind(this, function () {
-                                                  this._itemEntered(n);
-                                              }));
+        bbox.connect('clicked', Lang.bind(this, function() { this._onItemClicked(n); }));
+        bbox.connect('enter-event', Lang.bind(this, function() { this._onItemEnter(n); }));
 
         this._items.push(bbox);
     },
 
+    _onItemClicked: function (index) {
+        this._itemActivated(index);
+    },
+
+    _onItemEnter: function (index) {
+        this._itemEntered(index);
+    },
+
     addSeparator: function () {
         let box = new St.Bin({ style_class: 'separator' });
         this._separator = box;
@@ -609,16 +621,18 @@ SwitcherList.prototype = {
     },
 
     highlight: function(index, justOutline) {
-        if (this._highlighted != -1)
-            this._items[this._highlighted].style_class = 'item-box';
+        if (this._highlighted != -1) {
+            this._items[this._highlighted].remove_style_pseudo_class('outlined');
+            this._items[this._highlighted].remove_style_pseudo_class('selected');
+        }
 
         this._highlighted = index;
 
         if (this._highlighted != -1) {
             if (justOutline)
-                this._items[this._highlighted].style_class = 'outlined-item-box';
+                this._items[this._highlighted].add_style_pseudo_class('outlined');
             else
-                this._items[this._highlighted].style_class = 'selected-item-box';
+                this._items[this._highlighted].add_style_pseudo_class('selected');
         }
 
         let monitor = global.get_primary_monitor();
@@ -818,14 +832,14 @@ AppIcon.prototype = {
     }
 };
 
-function AppSwitcher(apps) {
-    this._init(apps);
+function AppSwitcher(apps, altTabPopup) {
+    this._init(apps, altTabPopup);
 }
 
 AppSwitcher.prototype = {
     __proto__ : SwitcherList.prototype,
 
-    _init : function(apps) {
+    _init : function(apps, altTabPopup) {
         SwitcherList.prototype._init.call(this, true);
 
         // Construct the AppIcons, sort by time, add to the popup
@@ -857,6 +871,8 @@ AppSwitcher.prototype = {
 
         this._curApp = -1;
         this._iconSize = 0;
+        this._altTabPopup = altTabPopup;
+        this._mouseTimeOutId = 0;
     },
 
     _getPreferredHeight: function (actor, forWidth, alloc) {
@@ -864,17 +880,19 @@ AppSwitcher.prototype = {
         while(this._items.length > 1 && this._items[j].style_class != 'item-box') {
                 j++;
         }
-        let iconPadding = this._items[j].get_theme_node().get_horizontal_padding();
+        let themeNode = this._items[j].get_theme_node();
+        let iconPadding = themeNode.get_horizontal_padding();
+        let iconBorder = themeNode.get_border_width(St.Side.LEFT) + themeNode.get_border_width(St.Side.RIGHT);
         let [iconMinHeight, iconNaturalHeight] = this.icons[j].label.get_preferred_height(-1);
-        let iconSpacing = iconNaturalHeight + iconPadding;
+        let iconSpacing = iconNaturalHeight + iconPadding + iconBorder;
         let totalSpacing = this._list.spacing * (this._items.length - 1);
         if (this._separator)
            totalSpacing += this._separator.width + this._list.spacing;
 
         // We just assume the whole screen here due to weirdness happing with the passed width
-        let focus = global.get_focus_monitor();
+        let primary = global.get_primary_monitor();
         let parentPadding = this.actor.get_parent().get_theme_node().get_horizontal_padding();
-        let availWidth = focus.width - parentPadding - this.actor.get_theme_node().get_horizontal_padding();
+        let availWidth = primary.width - parentPadding - this.actor.get_theme_node().get_horizontal_padding();
         let height = 0;
 
         for(let i =  0; i < iconSizes.length; i++) {
@@ -919,6 +937,29 @@ AppSwitcher.prototype = {
         }
     },
 
+    // We override SwitcherList's _onItemEnter method to delay
+    // activation when the thumbnail list is open
+    _onItemEnter: function (index) {
+        if (this._mouseTimeOutId != 0)
+            Mainloop.source_remove(this._mouseTimeOutId);
+        if (this._altTabPopup.thumbnailsVisible) {
+            this._mouseTimeOutId = Mainloop.timeout_add(APP_ICON_HOVER_TIMEOUT,
+                                                        Lang.bind(this, function () {
+                                                                            this._enterItem(index);
+                                                                            this._mouseTimeOutId = 0;
+                                                                            return false;
+                                                        }));
+        } else
+           this._itemEntered(index);
+    },
+
+    _enterItem: function(index) {
+        let [x, y, mask] = global.get_pointer();
+        let pickedActor = global.stage.get_actor_at_pos(Clutter.PickMode.ALL, x, y);
+        if (this._items[index].contains(pickedActor))
+            this._itemEntered(index);
+    },
+
     // We override SwitcherList's highlight() method to also deal with
     // the AppSwitcher->ThumbnailList arrows. Apps with only 1 window
     // will hide their arrows by default, but show them when their
