@@ -40,9 +40,18 @@ AlphabeticalView.prototype = {
         this.actor = new St.ScrollView({ x_fill: true,
                                          y_fill: false,
                                          y_align: St.Align.START,
-                                         vshadows: true });
+                                         vfade: true });
         this.actor.add_actor(box);
         this.actor.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC);
+        this.actor.connect('notify::mapped', Lang.bind(this,
+            function() {
+                if (!this.actor.mapped)
+                    return;
+
+                let adjustment = this.actor.vscroll.adjustment;
+                let direction = Overview.SwipeScrollDirection.VERTICAL;
+                Main.overview.setScrollAdjustment(adjustment, direction);
+            }));
     },
 
     _removeAll: function() {
@@ -281,21 +290,21 @@ PrefsSearchProvider.prototype = {
     }
 };
 
-function AppIcon(app) {
-    this._init(app);
+function AppIcon(app, params) {
+    this._init(app, params);
 }
 
 AppIcon.prototype = {
     __proto__:  IconGrid.BaseIcon.prototype,
 
-    _init : function(app) {
+    _init : function(app, params) {
         this.app = app;
 
         let label = this.app.get_name();
 
         IconGrid.BaseIcon.prototype._init.call(this,
                                                label,
-                                               { setSizeManually: true });
+                                               params);
     },
 
     createIcon: function(iconSize) {
@@ -303,12 +312,12 @@ AppIcon.prototype = {
     }
 };
 
-function AppWellIcon(app) {
-    this._init(app);
+function AppWellIcon(app, iconParams) {
+    this._init(app, iconParams);
 }
 
 AppWellIcon.prototype = {
-    _init : function(app) {
+    _init : function(app, iconParams) {
         this.app = app;
         this.actor = new St.Clickable({ style_class: 'app-well-app',
                                          reactive: true,
@@ -316,7 +325,7 @@ AppWellIcon.prototype = {
                                          y_fill: true });
         this.actor._delegate = this;
 
-        this.icon = new AppIcon(app);
+        this.icon = new AppIcon(app, iconParams);
         this.actor.set_child(this.icon.actor);
 
         this.actor.connect('clicked', Lang.bind(this, this._onClicked));
