@@ -78,8 +78,6 @@ function Dash() {
 
 Dash.prototype = {
     _init : function() {
-        this._menus = [];
-        this._menuDisplays = [];
         this._maxHeight = -1;
         this._iconSize = 48;
 
@@ -193,7 +191,8 @@ Dash.prototype = {
     },
 
     _addApp: function(app) {
-        let display = new AppDisplay.AppWellIcon(app);
+        let display = new AppDisplay.AppWellIcon(app,
+                                                 { setSizeManually: true });
         display._draggable.connect('drag-begin',
                                    Lang.bind(this, function() {
                                        display.actor.opacity = 50;
