@@ -22,6 +22,7 @@ const _ = Gettext.gettext;
 
 const Chrome = imports.ui.chrome;
 const CtrlAltTab = imports.ui.ctrlAltTab;
+const EndSessionDialog = imports.ui.endSessionDialog;
 const Environment = imports.ui.environment;
 const ExtensionSystem = imports.ui.extensionSystem;
 const MessageTray = imports.ui.messageTray;
@@ -39,6 +40,7 @@ const WindowManager = imports.ui.windowManager;
 const Magnifier = imports.ui.magnifier;
 const XdndHandler = imports.ui.xdndHandler;
 const StatusIconDispatcher = imports.ui.statusIconDispatcher;
+const Util = imports.misc.util;
 
 const DEFAULT_BACKGROUND_COLOR = new Clutter.Color();
 DEFAULT_BACKGROUND_COLOR.from_pixel(0x2266bbff);
@@ -65,6 +67,8 @@ let xdndHandler = null;
 let statusIconDispatcher = null;
 let _errorLogStack = [];
 let _startDate;
+let _defaultCssStylesheet = null;
+let _cssStylesheet = null;
 
 let background = null;
 
@@ -109,6 +113,7 @@ function start() {
     global.stage.color = DEFAULT_BACKGROUND_COLOR;
     global.stage.no_clear_hint = true;
 
+    _defaultCssStylesheet = global.datadir + '/theme/gnome-shell.css';
     loadTheme();
 
     let shellwm = global.window_manager;
@@ -170,6 +175,10 @@ function start() {
         }
     });
 
+    // Provide the bus object for gnome-session to
+    // initiate logouts.
+    EndSessionDialog.init();
+
     global.gdk_screen.connect('monitors-changed', _relayout);
 
     ExtensionSystem.init();
@@ -198,15 +207,44 @@ function start() {
     }
 }
 
+/**
+ * getThemeStylesheet:
+ *
+ * Get the theme CSS file that the shell will load
+ *
+ * Returns: A file path that contains the theme CSS,
+ *          null if using the default
+ */
+function getThemeStylesheet()
+{
+    return _cssStylesheet;
+}
+
+/**
+ * setThemeStylesheet:
+ * @cssStylesheet: A file path that contains the theme CSS,
+ *                  set it to null to use the default
+ *
+ * Set the theme CSS file that the shell will load
+ */
+function setThemeStylesheet(cssStylesheet)
+{
+    _cssStylesheet = cssStylesheet;
+}
+
 /**
  * loadTheme:
  *
- * Reloads the theme CSS file from the default theme.
+ * Reloads the theme CSS file
  */
 function loadTheme() {
     let themeContext = St.ThemeContext.get_for_stage (global.stage);
-    let stylesheetPath = global.datadir + '/theme/gnome-shell.css';
-    let theme = new St.Theme ({ application_stylesheet: stylesheetPath });
+
+    let cssStylesheet = _defaultCssStylesheet;
+    if (_cssStylesheet != null)
+        cssStylesheet = _cssStylesheet;
+
+    let theme = new St.Theme ({ application_stylesheet: cssStylesheet });
     themeContext.set_theme (theme);
 }
 
@@ -259,10 +297,8 @@ function _relayout() {
 
     // To avoid updating the position and size of the workspaces
     // in the overview, we just hide the overview. The positions
-    // will be updated when it is next shown. We do the same for
-    // the calendar popdown.
+    // will be updated when it is next shown.
     overview.hide();
-    panel.hideCalendar();
 }
 
 // metacity-clutter currently uses the same prefs as plain metacity,
@@ -317,11 +353,8 @@ function _globalKeyPressHandler(actor, event) {
     if (action == Meta.KeyBindingAction.COMMAND_SCREENSHOT) {
         let gconf = GConf.Client.get_default();
         let command = gconf.get_string('/apps/metacity/keybinding_commands/command_screenshot');
-        if (command != null && command != '') {
-            let [ok, len, args] = GLib.shell_parse_argv(command);
-            let p = new Shell.Process({'args' : args});
-            p.run();
-        }
+        if (command != null && command != '')
+            Util.spawnCommandLine(command);
         return true;
     }
 
@@ -371,6 +404,7 @@ function _findModal(actor) {
 /**
  * pushModal:
  * @actor: #ClutterActor which will be given keyboard focus
+ * @timestamp: optional timestamp
  *
  * Ensure we are in a mode where all keyboard and mouse input goes to
  * the stage, and focus @actor. Multiple calls to this function act in
@@ -381,11 +415,19 @@ function _findModal(actor) {
  * modal stack returns to this actor, reset the focus to the actor
  * which was focused at the time pushModal() was invoked.
  *
+ * @timestamp is optionally used to associate the call with a specific user
+ * initiated event.  If not provided then the value of
+ * global.get_current_time() is assumed.
+ *
  * Returns: true iff we successfully acquired a grab or already had one
  */
-function pushModal(actor) {
+function pushModal(actor, timestamp) {
+
+    if (timestamp == undefined)
+        timestamp = global.get_current_time();
+
     if (modalCount == 0) {
-        if (!global.begin_modal(global.get_current_time())) {
+        if (!global.begin_modal(timestamp)) {
             log('pushModal: invocation of begin_modal failed');
             return false;
         }
@@ -416,12 +458,21 @@ function pushModal(actor) {
 /**
  * popModal:
  * @actor: #ClutterActor passed to original invocation of pushModal().
+ * @timestamp: optional timestamp
  *
  * Reverse the effect of pushModal().  If this invocation is undoing
  * the topmost invocation, then the focus will be restored to the
  * previous focus at the time when pushModal() was invoked.
+ *
+ * @timestamp is optionally used to associate the call with a specific user
+ * initiated event.  If not provided then the value of
+ * global.get_current_time() is assumed.
  */
-function popModal(actor) {
+function popModal(actor, timestamp) {
+
+    if (timestamp == undefined)
+        timestamp = global.get_current_time();
+
     modalCount -= 1;
     let focusIndex = _findModal(actor);
     if (focusIndex >= 0) {
@@ -440,7 +491,7 @@ function popModal(actor) {
         return;
 
     global.stage.set_key_focus(null);
-    global.end_modal(global.get_current_time());
+    global.end_modal(timestamp);
     global.set_stage_input_mode(Shell.StageInputMode.NORMAL);
 }
 
