@@ -6,6 +6,7 @@ const Signals = imports.signals;
 
 const MessageTray = imports.ui.messageTray;
 const NotificationDaemon = imports.ui.notificationDaemon;
+const Util = imports.misc.util;
 
 const STANDARD_TRAY_ICON_IMPLEMENTATIONS = {
     'bluetooth-applet': 'bluetooth',
@@ -15,7 +16,10 @@ const STANDARD_TRAY_ICON_IMPLEMENTATIONS = {
     'nm-applet': 'network',
     'gnome-power-manager': 'battery',
     'keyboard': 'keyboard',
-    'gnome-settings-daemon': 'display'
+    'a11y-keyboard': 'a11y',
+    'kbd-scrolllock': 'keyboard',
+    'kbd-numlock': 'keyboard',
+    'kbd-capslock': 'keyboard'
 };
 
 function StatusIconDispatcher() {
@@ -33,8 +37,7 @@ StatusIconDispatcher.prototype = {
         // app-indicators, so that applications fall back to normal
         // status icons
         // http://bugzilla.gnome.org/show_bug.cgi=id=621382
-        let p = new Shell.Process({ args: ['pkill', '-f', '^([^ ]*/)?indicator-application-service$']});
-        p.run();
+        Util.killall('indicator-application-service');
     },
 
     _onTrayIconAdded: function(o, icon) {
