@@ -497,6 +497,8 @@ WindowManager.prototype = {
 
         for (let i = 0; i < switchData.windows.length; i++) {
                 let w = switchData.windows[i];
+                if (w.window.is_destroyed()) // Window gone
+                    continue;
                 if (w.window.get_parent() == switchData.outGroup) {
                     w.window.reparent(w.parent);
                     w.window.hide();
@@ -543,24 +545,34 @@ WindowManager.prototype = {
     },
 
     actionMoveWorkspaceLeft: function() {
+        let rtl = (St.Widget.get_default_direction() == St.TextDirection.RTL);
         let activeWorkspaceIndex = global.screen.get_active_workspace_index();
-        if (activeWorkspaceIndex > 0) {
-            global.screen.get_workspace_by_index(activeWorkspaceIndex - 1).activate(global.get_current_time());
-            if (!Main.overview.visible)
-                this._workspaceSwitcherPopup.display(WorkspaceSwitcherPopup.LEFT, activeWorkspaceIndex - 1);
-        } else if (!Main.overview.visible) {
-            this._workspaceSwitcherPopup.display(WorkspaceSwitcherPopup.LEFT, activeWorkspaceIndex);
-        }
+        let indexToActivate = activeWorkspaceIndex;
+        if (rtl && activeWorkspaceIndex < global.screen.n_workspaces - 1)
+            indexToActivate++;
+        else if (!rtl && activeWorkspaceIndex > 0)
+            indexToActivate--;
+
+        if (indexToActivate != activeWorkspaceIndex)
+            global.screen.get_workspace_by_index(indexToActivate).activate(global.get_current_time());
+
+        if (!Main.overview.visible)
+            this._workspaceSwitcherPopup.display(WorkspaceSwitcherPopup.LEFT, indexToActivate);
     },
 
     actionMoveWorkspaceRight: function() {
+        let rtl = (St.Widget.get_default_direction() == St.TextDirection.RTL);
         let activeWorkspaceIndex = global.screen.get_active_workspace_index();
-        if (activeWorkspaceIndex <  global.screen.n_workspaces - 1) {
-            global.screen.get_workspace_by_index(activeWorkspaceIndex + 1).activate(global.get_current_time());
-            if (!Main.overview.visible)
-                this._workspaceSwitcherPopup.display(WorkspaceSwitcherPopup.RIGHT, activeWorkspaceIndex + 1);
-        } else if (!Main.overview.visible) {
-            this._workspaceSwitcherPopup.display(WorkspaceSwitcherPopup.RIGHT, activeWorkspaceIndex);
-        }
+        let indexToActivate = activeWorkspaceIndex;
+        if (rtl && activeWorkspaceIndex > 0)
+            indexToActivate--;
+        else if (!rtl && activeWorkspaceIndex < global.screen.n_workspaces - 1)
+            indexToActivate++;
+
+        if (indexToActivate != activeWorkspaceIndex)
+            global.screen.get_workspace_by_index(indexToActivate).activate(global.get_current_time());
+
+        if (!Main.overview.visible)
+            this._workspaceSwitcherPopup.display(WorkspaceSwitcherPopup.RIGHT, indexToActivate);
     }
 };
