@@ -9,11 +9,12 @@
 #include <gio/gdesktopappinfo.h>
 #include <gtk/gtk.h>
 #include <clutter/clutter.h>
+#include <glib/gi18n.h>
 
 #include "shell-app-private.h"
 #include "shell-global.h"
-#include "st/st-texture-cache.h"
 #include "display.h"
+#include "st.h"
 
 #define GMENU_I_KNOW_THIS_IS_UNSTABLE
 #include <gmenu-tree.h>
@@ -1004,11 +1005,12 @@ shell_app_info_get_name (ShellAppInfo *info)
       return g_key_file_get_locale_string (info->keyfile, DESKTOP_ENTRY_GROUP, "Name", NULL, NULL);
     case SHELL_APP_INFO_TYPE_WINDOW:
       {
-        char *title;
-        g_object_get (info->window, "title", &title, NULL);
-        if (!title)
-          title = g_strdup ("");
-        return title;
+        const char *name;
+
+        name = meta_window_get_wm_class (info->window);
+        if (!name)
+          name = _("Unknown");
+        return g_strdup (name);
       }
   }
   g_assert_not_reached ();
@@ -1272,7 +1274,7 @@ shell_app_info_create_icon_texture (ShellAppInfo *info, float size)
     }
   else
     {
-      ret = st_texture_cache_load_gicon (st_texture_cache_get_default (), icon, (int)size);
+      ret = st_texture_cache_load_gicon (st_texture_cache_get_default (), NULL, icon, (int)size);
       g_object_unref (icon);
     }
 
@@ -1311,12 +1313,10 @@ shell_app_info_launch_full (ShellAppInfo *info,
                             GError      **error)
 {
   GDesktopAppInfo *gapp;
-  char *filename;
   GdkAppLaunchContext *context;
   gboolean ret;
   ShellGlobal *global;
   MetaScreen *screen;
-  MetaDisplay *display;
 
   if (startup_id)
     *startup_id = NULL;
@@ -1333,11 +1333,13 @@ shell_app_info_launch_full (ShellAppInfo *info,
     }
   else if (info->type == SHELL_APP_INFO_TYPE_ENTRY)
     {
-      gapp = g_desktop_app_info_new (shell_app_info_get_id (info));
+      /* Can't use g_desktop_app_info_new, see bug 614879 */
+      const char *filename = gmenu_tree_entry_get_desktop_file_path ((GMenuTreeEntry *)info->entry);
+      gapp = g_desktop_app_info_new_from_filename (filename);
     }
   else
     {
-      filename = shell_app_info_get_desktop_file_path (info);
+      char *filename = shell_app_info_get_desktop_file_path (info);
       gapp = g_desktop_app_info_new_from_filename (filename);
       g_free (filename);
     }
@@ -1350,7 +1352,6 @@ shell_app_info_launch_full (ShellAppInfo *info,
 
   global = shell_global_get ();
   screen = shell_global_get_screen (global);
-  display = meta_screen_get_display (screen);
 
   if (timestamp == 0)
     timestamp = clutter_get_current_event_time ();
