@@ -45,10 +45,11 @@ shell_cursor_get_type(void) {
   if (G_UNLIKELY (!enum_type_id))
     {
       static const GEnumValue values[] = {
-        { SHELL_CURSOR_DND_IN_DRAG, "SHELL_CURSOR_DND_IN_DRAG", "in-drag" },
-        { SHELL_CURSOR_DND_UNSUPPORTED_TARGET, "SHELL_CURSOR_DND_UNSUPPORTED_TARGET", "unsupported-target" },
-        { SHELL_CURSOR_DND_MOVE, "SHELL_CURSOR_DND_MOVE", "move" },
-        { SHELL_CURSOR_DND_COPY, "SHELL_CURSOR_DND_COPY", "copy" },
+        { SHELL_CURSOR_DND_IN_DRAG, "SHELL_CURSOR_DND_IN_DRAG", "dnd-in-drag" },
+        { SHELL_CURSOR_DND_UNSUPPORTED_TARGET, "SHELL_CURSOR_DND_UNSUPPORTED_TARGET", "dnd-unsupported-target" },
+        { SHELL_CURSOR_DND_MOVE, "SHELL_CURSOR_DND_MOVE", "dnd-move" },
+        { SHELL_CURSOR_DND_COPY, "SHELL_CURSOR_DND_COPY", "dnd-copy" },
+        { SHELL_CURSOR_POINTING_HAND, "SHELL_CURSOR_POINTING_HAND", "pointing-hand" },
         { 0, NULL, NULL }
       };
       enum_type_id = g_enum_register_static("ShellCursor", values);
